#include <string.h>
#include <stdio.h>
#include "Mydefs.h"
#include "mem_wrappers.h"
#include "html_parsing.h"
#include "errors.h"

char FromWhomClause[]="FromWhom=";
char SubjectClause[]="Subject=";
char TextClause[]="Text=";
char StatusClause[]="Status=";


int parse_query_string(	const char *query_string,  // <0 on error
			char **FromWhom, 
			char **Subject, 
			char **Text,
			char **Status)  {

	char buffer[MAX_STRING];
	int i, j, count;

	count=0;
	for(i=0, j=0;j<MAX_STRING && query_string[i]!=0 &&(count-4); i++, j++) {
		if (query_string[i]=='&') {
			count++;
			buffer[j]='\0';
			j=-1;
			switch (count) {
				case 1:
					*FromWhom=Strdup(buffer);
					break;
				case 2:
					*Subject=Strdup(buffer);
					break;
				case 3:
					*Text=Strdup(buffer);
					break;
				case 4: //This must not happen. JIC
					*Status=Strdup(buffer);
			} 
		}
		else {
			buffer[j]=query_string[i];
		}
	}
	if (query_string[i]=='\0') { 			// Everything OK
		count++;
		buffer[j]='\0';
		*Status=Strdup(buffer);
	}
	if (count!=4) {
		return (-1);			// Sort of args.
	}
	return (count);
}


int check_arguments(char *FromWhom, char *Subject, char *Text, char *Status) {

	int i;
	int check;

	check=1;
	for (i=0; i<strlen(FromWhomClause); i++) {
		if ( FromWhom[i] != FromWhomClause[i]) {
			check=-1;
		}
	}
	for (i=0; i<strlen(SubjectClause); i++) {
		if ( Subject[i] != SubjectClause[i]) {
			check-=2;
		}
	}
	for (i=0; i<strlen(TextClause); i++) {
		if ( Text[i] != TextClause[i]) {
			check-=4;
		}
	}
	for (i=0; i<strlen(StatusClause); i++) {
		if (Status[i] != StatusClause[i]) {
			check-=8;
		}
	}
	return(check);
}

int strip_arguments(char **FromWhom,char **Subject,char **Text,char **Status) {
	int i, skip, begin;
	int accept;

	accept=0;

	begin=0;
	skip=strlen(FromWhomClause);
	for(i=skip; i<strlen(*FromWhom); i++) {
		if ((*FromWhom)[i]!=' ' && (*FromWhom)[i]!='\t'
					&& (*FromWhom)[i]!='\0') {
			begin=1;
		}
		if (begin) {
			(*FromWhom)[i-skip]=(*FromWhom)[i];
		}
	}
	(*FromWhom)[i-skip]='\0';
	if(!begin) {
		free(*FromWhom);
		*FromWhom=Strdup("Anonymous Coward");
	}

	begin=0;
	skip=strlen(SubjectClause);
	for(i=skip; i<strlen(*Subject); i++) {
		if ( (*Subject)[i]!=' ' && (*Subject)[i]!='\t' 
					&& (*Subject)[i]!='\0') {
			begin=1;
		}
		(*Subject)[i-skip]=(*Subject)[i];
	}
	(*Subject)[i-skip]='\0';
	if(!begin) {
		free(*Subject);
		*Subject=Strdup("no subject");
	}
	else {	accept=1; }

	begin=0;
	skip=strlen(TextClause);
	for(i=skip; i<strlen(*Text); i++) {
		if ((*Text)[i]!=' ' && (*Text)[i]!='\t' && (*Text)[i]!='\0') {
			begin=1;
		}
		(*Text)[i-skip]=(*Text)[i];
	}
	(*Text)[i-skip]='\0';
	if(!begin) {
		free(*Text);
		*Text=Strdup("no message body");
	}
	else {  accept=1; }

	begin=0;
	skip=strlen(StatusClause);
	for(i=skip; i<strlen(*Status); i++) {
		if ( (*Status)[i] !=' ' && (*Status)[i]!='\t' 
					&& (*Status)[i]!='\0') {
			begin=1;
		}
		(*Status)[i-skip]=(*Status)[i];
	}
	(*Status)[i-skip]='\0';
	if(!begin) {
		free(*Status);
		*Status=Strdup("__INVALID__");
	}
	else {
		accept=-1;
	}

	return(accept);
}

void cgi_to_text_arguments(char **FromWhom, 
			   char **Subject, 
			   char **Text,
			   char **Status) {
	char *temp;

	temp=*FromWhom;
	*FromWhom=cgi_to_text(temp);
	free(temp);

	temp=*Subject;
	*Subject=cgi_to_text(temp);
	free(temp);

	temp=*Text;
	*Text=cgi_to_text(temp);
	free(temp);

	temp=*Status;
	*Status=cgi_to_text(temp);
	free(temp);
}

process_arguments ( char *query_string, char **FromWhom,
					char **Subject,
					char **Text,
					char **Status) {

	if(parse_query_string(query_string, FromWhom, Subject, Text, Status)<0){
		fatal_error("Wrong query format");
	}
	printf("From: %s <BR>", cgi_to_text(*FromWhom));
	printf("Subject: %s <BR>", cgi_to_text(*Subject));
	printf("Message: %s <BR>", cgi_to_text(*Text));
	printf("Status: %s <BR>", cgi_to_text(*Status));
	cgi_to_text_arguments(FromWhom, Subject, Text, Status);
	if(check_arguments(*FromWhom, *Subject, *Text, *Status)<=0) {
		fatal_error("Wrong format. Naughty boy");
	}
	if(strip_arguments(FromWhom, Subject, Text, Status)==0) {
		fatal_error("No data in message");
	}
}

