#include <stdio.h>
#include <string.h>
#include "Mydefs.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include "various_functions.h"
#include "mem_wrappers.h"
#include "msg_formating.h"

char *get_filename(const char *msg_no);
off_t get_position(int fd, const char *msg_no); 

int update_field(const char *msg_no, int field, const char *field_value) {
	int fd;
	off_t position;

	if ( (fd=open(get_filename(msg_no), O_RDWR)) <0) {
		return (-1);
	}
	if((position=get_position(fd, msg_no))<0) {
		close(fd);
		return(-1);
	}
	lseek(fd, position, SEEK_SET);
	position=MAX_SIZE_OF_REST;
	switch (field) {
		case PARENT_FIELD:
			position+=0 * SIZE_OF_MSG_NO ;
			break;
		case NEXT_FIELD:
			position+=1 * SIZE_OF_MSG_NO ;
			break;
		case FIRST_REPLY_FIELD:
			position+=2 * SIZE_OF_MSG_NO ;
			break;
	}
	if(lseek(fd, position, SEEK_CUR)<0) {
		close(fd);
		return(-1);
	}
	if(write(fd, field_value, SIZE_OF_MSG_NO)<SIZE_OF_MSG_NO) {
		close(fd);
		return(-1);
	}
	close(fd);
	return(1);
}

char *get_field(const char *msg_no, int field) {
	int fd;
	off_t position;
	char *return_value;

	if ( (fd=open(get_filename(msg_no), O_RDWR)) <0) {
		return (NULL);
	}
	if((position=get_position(fd, msg_no))<0) {
		close(fd);
		return(NULL);
	}
	lseek(fd, position, SEEK_SET);
	position=MAX_SIZE_OF_REST;
	switch (field) {
		case PARENT_FIELD:
			position+=0 * SIZE_OF_MSG_NO ;
			break;
		case NEXT_FIELD:
			position+=1 * SIZE_OF_MSG_NO ;
			break;
		case FIRST_REPLY_FIELD:
			position+=2 * SIZE_OF_MSG_NO ;
			break;
	}
	if(lseek(fd, position, SEEK_CUR)<0) {
		close(fd);
		return(NULL);
	}
	return_value=(char *)Malloc(SIZE_OF_MSG_NO+1);
	if(read(fd, return_value, SIZE_OF_MSG_NO)<SIZE_OF_MSG_NO) {
		free(return_value);
		close(fd);
		return(NULL);
	}
	return_value[SIZE_OF_MSG_NO]='\0';
	close(fd);
	return(return_value);
}

char *get_msg(const char *msg_no) {	// Memory leak here (in open)
	int fd, n;
	char cur_msg_no[SIZE_OF_MSG_NO+1];
	char size_of_rest[MAX_SIZE_OF_REST+1];
	int size_value;
	off_t position;
	char *message;

	if ( (fd=open(get_filename(msg_no), O_RDONLY)) <0) {
		return(NULL);
	}
	if ((position=get_position(fd, msg_no))<0) {
		close(fd);
		return(NULL);
	}
	lseek(fd, position, SEEK_SET);
	if(read(fd, size_of_rest, MAX_SIZE_OF_REST)<MAX_SIZE_OF_REST) {
		close(fd);
		return(NULL);
	}
	size_of_rest[MAX_SIZE_OF_REST]='\0';
	sscanf(size_of_rest, "%d", &size_value);
	message=(char *)Malloc(size_value+1);
	if(read(fd, message, size_value)<size_value) {
		close(fd);
		return(NULL);
	}
	message[size_value]='\0';
	close(fd);
	return(message);
}

int write_msg(char *msg_no, char *message) {
	int fd;
	char size_of_rest[MAX_SIZE_OF_REST+1];
	int i, j, real_size;
	char *filename;
	
	if ( (fd=open(filename=(get_filename(msg_no)), O_RDWR)) < 0) {
		if( (fd=creat(filename, 0755)) <0) {
			return(-1);
		}
		if(put_null_header(fd)<0) {
			return(-1);
		}
	}
	lseek(fd, 0, SEEK_END);
	if(perfect_fit(size_of_rest, strlen(message), MAX_SIZE_OF_REST)<0) {
		close(fd);
		return(-1);
	}
	if(write(fd, msg_no, SIZE_OF_MSG_NO) < SIZE_OF_MSG_NO) {
		close(fd);
		return(-1);
	}
	if(write(fd, size_of_rest, MAX_SIZE_OF_REST)<MAX_SIZE_OF_REST) {
		close(fd);
		return(-1);
	}
	if(write(fd, message, strlen(message)) < strlen(message)) {
		close(fd);
		return(-1);
	}
	return(1);
}

char *get_filename(const char *msg_no) {
	char *buf;
	buf=(char *)Malloc(strlen(MSG_FOLDER)+SIZE_OF_MSG_FILE+1);
	strcpy(buf, MSG_FOLDER);
	strncat(buf, msg_no, SIZE_OF_MSG_FILE);
	return(buf);
}

off_t get_position(int fd, const char *msg_no) {
	char cur_msg_no[SIZE_OF_MSG_NO+1];
	char size_of_rest[MAX_SIZE_OF_REST+1];	
	int size_value;

	lseek(fd, 2*SIZE_OF_MSG_NO, SEEK_SET);
	while(1) {
		if (read(fd, cur_msg_no, SIZE_OF_MSG_NO) < SIZE_OF_MSG_NO) {
			return(-1);
		}
		cur_msg_no[SIZE_OF_MSG_NO]='\0';
		if (!strncmp(cur_msg_no, msg_no, SIZE_OF_MSG_NO)) { // FOUND
			return (lseek(fd, 0, SEEK_CUR));
		}
		else {
			if(read(fd, size_of_rest, MAX_SIZE_OF_REST)<MAX_SIZE_OF_REST) {
				return(-1);
			}
			size_of_rest[MAX_SIZE_OF_REST]='\0';
			sscanf(size_of_rest, "%d", &size_value);
			if( lseek(fd, size_value, SEEK_CUR) <0 ) {
				return(-1);
			}
		}
	}
}

char *read_first_of_main(int fd) {
	char *return_value;
	if(lseek(fd, 0, SEEK_SET)<0) {
		return (NULL);
	}
	return_value=(char *)Malloc(SIZE_OF_MSG_NO+1);
	if(read(fd, return_value, SIZE_OF_MSG_NO)<SIZE_OF_MSG_NO) { 
		return(NULL);
	}
	return_value[SIZE_OF_MSG_NO]='\0';
	return(return_value);
}

char *get_last_reply(const char *parent) {
	char *return_value;
	char *tmp_buf;

	if((tmp_buf=get_field(parent, FIRST_REPLY_FIELD))==NULL) {
		return(NULL);
	}
	tmp_buf[SIZE_OF_MSG_NO]='\0';
	return_value=(char *)Malloc(SIZE_OF_MSG_NO+1);
	strncpy(return_value, parent, SIZE_OF_MSG_NO);
	return_value[SIZE_OF_MSG_NO]='\0';
	while(! field_is_null(tmp_buf, SIZE_OF_MSG_NO)) {
		strncpy(return_value, tmp_buf, SIZE_OF_MSG_NO);
		free(tmp_buf);
		if((tmp_buf=get_field(return_value, NEXT_FIELD))==NULL) {
			free(tmp_buf);
			return(NULL);
		}
	}
	free(tmp_buf);
	return(return_value);
}

int put_first_of_main(int fd, char *msg_no) {
	if(lseek(fd, 0, SEEK_SET)<0) {
		return (-1);
	}
	if(write(fd, msg_no, SIZE_OF_MSG_NO)<SIZE_OF_MSG_NO) {
		return (-1);
	}
	return(1);
}

char *read_first_of_stray(int fd) {
	char *return_value;
	if(lseek(fd, SIZE_OF_MSG_NO, SEEK_SET)<SIZE_OF_MSG_NO) {
		return (NULL);
	}
	return_value=(char *)Malloc(SIZE_OF_MSG_NO+1);
	if(read(fd, return_value, SIZE_OF_MSG_NO)<SIZE_OF_MSG_NO) { 
		return(NULL);
	}
	return_value[SIZE_OF_MSG_NO]='\0';
	return(return_value);
}


int put_first_of_stray(int fd, char *msg_no) {
	if(lseek(fd, SIZE_OF_MSG_NO, SEEK_SET)<SIZE_OF_MSG_NO) {
		return (-1);
	}
	if(write(fd, msg_no, SIZE_OF_MSG_NO)<SIZE_OF_MSG_NO) {
		return (-1);
	}
	return(1);
}

int get_data_from_message(const char *message, 
				char **FromWhom, 
				char **Subject, 
				char **Text) {
	
	char pro_buf[20];
	int cur_pos=0, size_of_field;

	strncpy(pro_buf, message+3*SIZE_OF_MSG_NO, MAX_SIZE_OF_WHO);
	pro_buf[MAX_SIZE_OF_WHO]='\0';
	sscanf(pro_buf, "%d", &size_of_field);
	cur_pos=3*SIZE_OF_MSG_NO+MAX_SIZE_OF_WHO;
	*FromWhom=(char *)Malloc(size_of_field+1);
	strncpy(*FromWhom, message+cur_pos, size_of_field);
	(*FromWhom)[size_of_field]='\0';
	cur_pos+=size_of_field;

	strncpy(pro_buf, message+cur_pos, MAX_SIZE_OF_SUBJECT);
	pro_buf[MAX_SIZE_OF_SUBJECT]='\0';
	sscanf(pro_buf, "%d", &size_of_field);
	cur_pos+=MAX_SIZE_OF_SUBJECT;
	*Subject=(char *)Malloc(size_of_field+1);
	strncpy(*Subject, message+cur_pos, size_of_field);
	(*Subject)[size_of_field]='\0';
	cur_pos+=size_of_field;

	strncpy(pro_buf, message+cur_pos, MAX_SIZE_OF_TEXT);
	pro_buf[MAX_SIZE_OF_TEXT]='\0';
	sscanf(pro_buf, "%d", &size_of_field);
	cur_pos+=MAX_SIZE_OF_TEXT;
	*Text=(char *)Malloc(size_of_field+1);
	strncpy(*Text, message+cur_pos, size_of_field);
	(*Text)[size_of_field]='\0';

	return(1);
}

